#include "PS2.h"
#include <SoftwareSerial.h>
#include "Teds_PS2X_lib.h"  //for v1.6
#define DAT 6
#define CMD 7
#define ATT 8

// Tried this.  Worked OK for polling but not for interrupts
//#define DAT 2
//#define CMD 3
//#define ATT 4

// Tried this.  Now nothing works!
//#define DAT 4
//#define CMD 3
//#define ATT 2

#define CLK 9      // BotBoarduino pins used for the PS2 controller
// These pin numbers must be the ones actually used in your BotBoarduino 

//right now, the library does NOT support hot pluggable controllers, meaning 
//you must always either restart your Arduino after you conect the controller, 
//or call config_gamepad(pins) again after connecting the controller.
int error = 0; 
int type = 0;
int vibrate = 0;

PS2X ps2x; // create PS2 Controller Class

//###########################################################################
// Function to read and return ID of button/switch pressed 
//###########################################################################
int readPS2(boolean debug){
    ps2x.read_gamepad();
    
    if(ps2x.Button(PSB_START)){                   //will be TRUE as long as button is pressed
      if(debug) Serial.println("Start is being held");
      return PSB_START;
         }
    if(ps2x.Button(PSB_SELECT)){
      if(debug) Serial.println("Select is being held");
      return PSB_SELECT;
      }                 
    if(ps2x.Button(PSB_PAD_UP)) {         //will be TRUE as long as button is pressed
       if(debug) Serial.print("Up held this hard: ");
       Serial.println(ps2x.Analog(PSAB_PAD_UP), DEC);
       return PSB_PAD_UP;
       }
    if(ps2x.Button(PSB_PAD_RIGHT)){
       if(debug) Serial.print("Right held this hard: ");
       if(debug) Serial.println(ps2x.Analog(PSAB_PAD_RIGHT), DEC);
       return PSB_PAD_RIGHT;
       }         
     if(ps2x.Button(PSB_PAD_LEFT)){
       if(debug) Serial.print("LEFT held this hard: ");
       if(debug) Serial.println(ps2x.Analog(PSAB_PAD_LEFT), DEC);
       return PSB_PAD_LEFT;
       }         
     if(ps2x.Button(PSB_PAD_DOWN)){
       if(debug) Serial.print("DOWN held this hard: ");
       if(debug) Serial.println(ps2x.Analog(PSAB_PAD_DOWN), DEC);
       return PSB_PAD_DOWN;
       }         
     if (ps2x.NewButtonState()){               //will be TRUE if any button changes state (on to off, or off to on)         
        if(ps2x.Button(PSB_L3)){
          if(debug) Serial.println("L3 pressed");
          return PSB_L3;
          }         
        if(ps2x.Button(PSB_R3)){
          if(debug) Serial.println("R3 pressed");
          return PSB_R3;
          }         
        if(ps2x.Button(PSB_L2)){
          if(debug) Serial.println("L2 pressed");
          return PSB_L2;
          }         
        if(ps2x.Button(PSB_R2)){
          if(debug) Serial.println("R2 pressed");
          return PSB_R2;
          }         
        if(ps2x.Button(PSB_TRIANGLE)){
          if(debug) Serial.println("Triangle pressed");
          return PSB_TRIANGLE;
          }                  
        if(ps2x.Button(PSB_CIRCLE)){
          if(debug) Serial.println("Circle pressed");
          return PSB_CIRCLE;
          }                  
        if(ps2x.Button(PSB_CROSS)){
          if(debug) Serial.println("Cross pressed");
          return PSB_CROSS;
          }                  
        if(ps2x.Button(PSB_SQUARE)){
          if(debug) Serial.println("Square pressed");
          return PSB_SQUARE;
          }                  
        if(ps2x.Button(PSB_L1)){
          if(debug) Serial.println("L1 pressed");
          return ps2x.Analog(PSS_LX); //Left stick, X axis.
          }
        if(ps2x.Button(PSB_L2)){
          if(debug) Serial.println("L2 pressed");
          return ps2x.Analog(PSS_LY); //Left stick, Y axis.  
          }
        if(ps2x.Button(PSB_R1)){
          if(debug) Serial.println("R1 pressed");
          return ps2x.Analog(PSS_RX); //Right stick, X axis.  
          }
        if(ps2x.Button(PSB_R2)){
          if(debug) Serial.println("R2 pressed");
          return ps2x.Analog(PSS_RY); //Right stick, Y axis.  
          }
      }   
    return 0;    // Gets to here only if nothing pressed
 }
         
void setupPS2(){
  error = ps2x.config_gamepad(CLK, CMD, ATT, DAT, true, true);   //setup pins and settings:  GamePad(clock, command, attention, data, Pressures?, Rumble?) check for error
 
 if(error == 0){
   Serial.println("Found Controller, configured successful");
//   Serial.println("Try out all the buttons, X will vibrate the controller, faster as you press harder;");
//   Serial.println("holding L1 or R1 will print out the analog stick values.");
//   Serial.println("Go to www.billporter.info for updates and to report bugs.");
   }   
  else if(error == 1)
   Serial.println("No controller found, check wiring, see readme.txt to enable debug. visit www.billporter.info for troubleshooting tips"); 
  else if(error == 2)
   Serial.println("Controller found but not accepting commands. see readme.txt to enable debug. Visit www.billporter.info for troubleshooting tips");   
  else if(error == 3)
   Serial.println("Controller refusing to enter Pressures mode, may not support it. ");
   
   //Serial.print(ps2x.Analog(1), HEX);
   
   type = ps2x.readType(); 
     switch(type) {
       case 0:
         Serial.println("Unknown Controller type");
         break;
       case 1:
         Serial.println("DualShock Controller Found");
         break;
       case 2:
         Serial.println("GuitarHero Controller Found");
         break;
       }
  }

/*    
    if(ps2x.ButtonPressed(PSB_RED))             //will be TRUE if button was JUST pressed
         if(debug) Serial.println("Circle just pressed");
         
    if(ps2x.ButtonReleased(PSB_PINK))             //will be TRUE if button was JUST released
         if(debug) Serial.println("Square just released");     
    
    if(ps2x.NewButtonState(PSB_BLUE))            //will be TRUE if button was JUST pressed OR released
         if(debug) Serial.println("X just changed");    
    
    
    if(ps2x.Button(PSB_L1) || ps2x.Button(PSB_R1)) // print stick values if either is TRUE
    {
        if(debug) Serial.print("Stick Values:");
        if(debug) Serial.print(ps2x.Analog(PSS_LY), DEC); //Left stick, Y axis. Other options: LX, RY, RX  
        if(debug) Serial.print(",");
        if(debug) Serial.print(ps2x.Analog(PSS_LX), DEC); 
        if(debug) Serial.print(",");
        if(debug) Serial.print(ps2x.Analog(PSS_RY), DEC); 
        if(debug) Serial.print(",");
        if(debug) Serial.println(ps2x.Analog(PSS_RX), DEC); 
    } 
*/    
 
/*
int readPS2(){
   
//  Serial.println("readPS2 called.");
  
  if(error == 1) // No controller found
  return 0; 
   
 else { //DualShock Controller
  
//    ps2x.read_gamepad(false, vibrate);          //read controller and set large motor to spin at 'vibrate' speed
    
    ps2x.read_gamepad();
    if(ps2x.Button(PSB_START))                   //will be TRUE as long as button is pressed
         Serial.println("Start is being held");
         return PSB_START;
    if(ps2x.Button(PSB_SELECT))
         Serial.println("Select is being held");
         return PSB_SELECT;
         
         
     if(ps2x.Button(PSB_PAD_UP)) {         //will be TRUE as long as button is pressed
       Serial.print("Up held this hard: ");
       Serial.println(ps2x.Analog(PSAB_PAD_UP), DEC);
       return PSB_PAD_UP;
      }
      if(ps2x.Button(PSB_PAD_RIGHT)){
        Serial.print("Right held this hard: ");
        Serial.println(ps2x.Analog(PSAB_PAD_RIGHT), DEC);
        return PSB_PAD_RIGHT;
      }
      if(ps2x.Button(PSB_PAD_LEFT)){
        Serial.print("LEFT held this hard: ");
        Serial.println(ps2x.Analog(PSAB_PAD_LEFT), DEC);
      }
      if(ps2x.Button(PSB_PAD_DOWN)){
        Serial.print("DOWN held this hard: ");
        Serial.println(ps2x.Analog(PSAB_PAD_DOWN), DEC);
        return PSB_PAD_DOWN;
      }   
  
    
      vibrate = ps2x.Analog(PSAB_BLUE);        //this will set the large motor vibrate speed based on 
                                              //how hard you press the blue (X) button    
    
    if (ps2x.NewButtonState())               //will be TRUE if any button changes state (on to off, or off to on)
    {         
        if(ps2x.Button(PSB_L1)){
          Serial.println("L1 pressed");
          return PSB_L1;
          }
        if(ps2x.Button(PSB_L2)){
          Serial.println("L2 pressed");
          return PSB_L2;
          }
        if(ps2x.Button(PSB_L3)){
          Serial.println("L3 pressed");
          return PSB_L3;
          }
        if(ps2x.Button(PSB_R1)){
          Serial.println("R1 pressed");
          return PSB_R1;
          }
        if(ps2x.Button(PSB_R2)){
          Serial.println("R2 pressed");
          return PSB_R2;
          }
        if(ps2x.Button(PSB_R3)){
          Serial.println("R3 pressed");
          return PSB_R3;
          }
        if(ps2x.Button(PSB_GREEN)){
          Serial.println("Triangle pressed");
          return PSB_GREEN;
          }
    }   
            
    if(ps2x.ButtonPressed(PSB_RED)){             //will be TRUE if button was JUST pressed
      Serial.println("Circle just pressed");
      return PSB_RED;
      }
    if(ps2x.ButtonReleased(PSB_PINK)){             //will be TRUE if button was JUST released
      Serial.println("Square just released");     
      return PSB_PINK;
      }   
    if(ps2x.NewButtonState(PSB_BLUE)){            //will be TRUE if button was JUST pressed OR released
      Serial.println("X just changed");    
      return PSB_BLUE;
      }
    
    if(ps2x.Button(PSB_L1) || ps2x.Button(PSB_R1)){ // print stick values if either is TRUE
        Serial.print("Stick Values:");
        Serial.print(ps2x.Analog(PSS_LY), DEC); //Left stick, Y axis. Other options: LX, RY, RX  
        Serial.print(", ");
        Serial.print(ps2x.Analog(PSS_LX), DEC); 
        Serial.print(", ");
        Serial.print(ps2x.Analog(PSS_RY), DEC); 
        Serial.print(", ");
        Serial.println(ps2x.Analog(PSS_RX), DEC); 
        } 
    }
 
 
//    delay(50);
     
  }
*/

