// Use an interrupt to read the PS2.  Did not work.

#include <avr/pgmspace.h>    // For using PROGMEM
#include <AVDFlash.h>
#include "Definitions.h"
#include "PS2.h"
#include "Teds_PS2X_lib.h"  //for v1.6
#include <SoftwareSerial.h>      // Comment this out if using hardware serial

#define SWSerial    // Comment this out if using hardware serial
#ifdef SWSerial
#include <SoftwareSerial.h>
#define cSSC_OUT     12         //Output pin for (SSC32 RX) on BotBoard (Yellow)
#define cSSC_IN      13         //Input pin for (SSC32 TX) on BotBoard (Blue)
#define cSSC_BAUD        38400   //SSC32 BAUD rate
SoftwareSerial SSCSerial(cSSC_IN, cSSC_OUT);  // Comment this line out if using hardware serial
#else
#define SSCSerial Serial    // Use hardware serial
#endif

#define usePS2    // Comment this line out if not using PS2
#ifdef usePS2
void setupPS2();
int readPS2(boolean);
#endif

#define interruptOnPin_2   0   // Interrupt 0 is on pin 2
#define interruptOnPin_3   1   // Interrupt 1 is on pin 3

int STRAIGHT = 1;
int TURN = 2;
int gait = STRAIGHT;    // Initial gait
int phaseTime = 500;    // Time (ms) for servos to complete a phase
int maxTime = 2000;  // Maximum allowable time per phase.  Can be as large as desired, for slowest motion.
int minTime = 100;    // Minimum allowable time per phase.  Must be sufficient,  not too small.
float timeFactor = 1.5;
unsigned long startTime, previousTime, extraTime, currentTime;
unsigned long computeTime, timeIN, timeOUT;  // Start time, finish time & time difference for calculations
int input;          // Input from PS2 controller
float pi = 3.14159;
float muSecPerRad = 2000/pi;  //2000 muSec pulse width = 180 deg (pi radians)
float radPerDeg = pi/180;

float H = 13.;    // Distance from hip horizontal axis to ground, mm
float deltaH = 1.;  // Correction for vertical sag when on three legs, mm
float d = 8.5;    // Distance from hip vertical axis out to path on ground (Hip at reference position), mm
float L0 = 3.;    // Distance from hip vertical axis to hip horizontal axis, mm
float L1 = 5.7;   // Distance from hip horizontal axis to knee axis, mm
float L2 = 13.;   // Distance from knee axis to end of leg (length of leg), mm
float r = 13.5;      // Radius from robot center to hip vertical axis, cm
float R = r + d;     // Radius from robot center to knee vertical axis, cm
float liftAngle = 30;    //hip vertical lift angle for lifted legs (deg)
//float hipUp = -liftAngle*radPerDeg;    // Convert liftAngle to radians
float alfaSTRAIGHT; //Maximum value of hip horizontal swing angle that balances OK
float alfaTURN;
float alfa1;        // First angle of hip about vertical axis
float alfa2;        // Second angle of hip about vertical axis
float gammaSTRAIGHT = 20; // Angle hip rotates about robot's vertical axis for each straight step, deg
float gammaTURN = 10;  //Angle robot rotates about robot's vertial axis for each turn step, deg
int initialPhase = 0, deltaPhase = 1;  // Initial values.  Commands can change these.
float legAngle = 60;  // Angle from robot center to leg hip vertical axis
boolean start = false;    // Initially, wait for START sigle
#define nPhases 8        // Number of gait phases
#define nLegs   6        // Number of legs on robot
#define up 1    // up
#define dn 0	// down

static const byte gait_positions[nPhases][nLegs] = {
	{dn, up, dn, up, dn, up},  //0
	{dn, up, dn, up, dn, up},  //1
        {dn, dn, dn, dn, dn, dn},  //2
	{up, dn, up, dn, up, dn},  //3
	{up, dn, up, dn, up, dn},  //4
	{up, dn, up, dn, up, dn},  //5
	{dn, dn, dn, dn, dn, dn},  //6
	{dn, up, dn, up, dn, up},  //7
	};
	
byte    gaitAngleIndices[nPhases][nLegs] = {  // i values for angles[i] function for each phase each leg
	{2, 2, 2, 2, 2, 2},  //0, servos in reference positions (alfa = 0)
	{3, 1, 3, 1, 3, 1},  //1
	{4, 0, 4, 0, 4, 0},  //2
	{3, 1, 3, 1, 3, 1},  //3
	{2, 2, 2, 2, 2, 2},  //4, reference positions
	{1, 3, 1, 3, 1, 3},  //5
	{0, 4, 0, 4, 0, 4},  //6
	{1, 3, 1, 3, 1, 3},  //7
	};

byte    gait_2_signs[nPhases][nLegs] = {  // i values for angles[i] function for each phase each leg
	{ 0,  0,  0,  0,  0,  0},  //0, servos in reference positions (alfa = 0)
	{ 1,  1,  1,  1,  1,  1},  //1
	{ 0,  0,  0,  0,  0,  0},  //2
	{-1, -1, -1, -1, -1, -1},  //3
	{ 0,  0,  0,  0,  0,  0},  //4, reference positions (alfa = 0); all legs down
	{ 1,  1,  1,  1,  1,  1},  //5
	{ 0,  0,  0,  0,  0,  0},  //6
	{-1, -1, -1, -1, -1, -1},  //7
	};

leg_servo_inputs leg[nLegs];  // Store IK results in this struct array(Definitions.h)

// Servo connections on SSC-32
byte servo[nLegs][3] = 
	{{0, 1, 2}, {4, 5, 6}, {8, 9, 10}, {16, 17, 18}, {20, 21, 22}, {24, 25, 26}};

// Actual calibrated servoRefs will be read from the SSC-32 initial-pulse-width registers
// and stored here using FindServoInitialSettings().
int servoRefs[nLegs][3] = {
	{1500, 1500, 1500}, {1500, 1500, 1500}, {1500, 1500, 1500},
	{1500, 1500, 1500}, {1500, 1500, 1500}, {1500, 1500, 1500}};
					
// These are the same as above except for having 60 deg added to thr corner legs
int turningServoRefs[nLegs][3] = {
	{1500, 1500, 1500}, {1500, 1500, 1500}, {1500, 1500, 1500},
	{1500, 1500, 1500}, {1500, 1500, 1500}, {1500, 1500, 1500}};
					
void FindServoInitialSettings() {  // Read initial pulse widths from SSC-32 
  byte szTemp[5];
  byte cbRead;
  int data;
  int ich;
  int delta = (int)legAngle*radPerDeg*muSecPerRad;    
  for (int i = 0; i < nLegs; i++) {    // Six legs      
    for (int j = 0; j < 3; j++) {  // Three servos per leg
      SSCSerial.print("R");
      SSCSerial.println(64 + servo[i][j], DEC);  // Register 64, start of initial pulse widths
      // now read in the current value
      cbRead = SSCRead((byte*)szTemp, sizeof(szTemp), 10000, 13);
      if (cbRead > 0){
        servoRefs[i][j] = atoi((const char *)szTemp);
        if (j == 0){
          if(i == 0 || i == 5){
            turningServoRefs[i][j] = servoRefs[i][j] - delta;
            }
          if(i == 1 || i == 4){
            turningServoRefs[i][j] = servoRefs[i][j];
            }
          if(i == 2 || i == 3){
            turningServoRefs[i][j] = servoRefs[i][j] + delta;
            }
          }
        else turningServoRefs[i][j] = servoRefs[i][j];
        }
        else Serial.println("SSCRead failed!");
      }
    }
  }

// This is the function to use to calculate D when turning.
// Seek value of d making this function = 0.  Use Newton's method to seek the solution
float function(float d, float alfa){
  float temp = sin(alfa);
//  float f = d*d + 2*r*sqrt(R*R - d*d*temp*temp) - R*R - r*r;
  float f = r + d*cos(alfa) - sqrt(R*R - d*d*temp*temp);
  return f;
  }

// Iterative method for finding value of d making function(d, alfa) equal to zero
float newtonsMethod(float d, float alfa){;  // Iterative method for finding d making function = 0
  boolean done = false;
  float fPrime;
  float epsilon = 0.1;    // Acceptable error, mm
  float trial_d, new_d;
  float referenceFunction;
  trial_d = d/cos(alfa);  // Initial estimate
  while(!done){
    Serial.println("In newtonsMethod.");
    referenceFunction = function(trial_d, alfa);
    fPrime = (function(trial_d + epsilon, alfa) - referenceFunction)/epsilon;  // derivative
    new_d = trial_d - referenceFunction/fPrime;  // Extrapolate to approximate zero-value point
    Serial.print("new_d = ");
    Serial.println(new_d);
    if(abs(new_d - trial_d) <= epsilon) done = true;  // Stop if close enough
    else trial_d = new_d;    // Try again
    }
  return new_d;
  }
  
// D is the ground-plane distance from the hip vertical axis to the foot of the leg
float calculateD(float alfa){
  if(gait == STRAIGHT){Serial.println("In calculateD, gait == STRAIGHT"); return d/cos(alfa);}
  if(gait == TURN){Serial.println("In calculateD, gait == TURN"); return newtonsMethod(d, alfa);}
  }

// Just a convenience
int ServoPosition(int i, int j, int value){
  if(gait == STRAIGHT) return (int)(servoRefs[i][j] + value);
  if(gait == TURN) return (int)(turningServoRefs[i][j] + value);
  }


// Sends a servo-position command to the SSC-32
void buildCommand(int i, int j, int value){  // Command for the SSC-32
//  return;  // For testing, no servo action
  SSCSerial.print(" #");
  SSCSerial.print(servo[i][j]);
  SSCSerial.print(" P");
  SSCSerial.print(ServoPosition(i, j, value));
  }

//==============================================================================
// Function to read bytes from the SSC-32 with a timeout and an end of character marker...
// Thanks to LynxMotion, and to Kurt Eckhardt and Jeroen Janssen
//==============================================================================
int SSCRead (byte* pb, int cb, word wTimeout, word wEOL)
{
    int ich;
    byte* pbIn = pb;
    unsigned long ulTimeLastChar = micros();
    while (cb) {
        while (!SSCSerial.available()) {
            // check for timeout
            if ((word)(micros()-ulTimeLastChar) > wTimeout) {
                return (int)(pb-pbIn);
            }    
        }
        ich = SSCSerial.read();
        *pb++ = (byte)ich;
        cb--;

        if ((word)ich == wEOL)
            break;    // we matched so get out of here.
        ulTimeLastChar = micros();    // update to say we received something
    }

    return (int)(pb-pbIn);
}

float getAlfa(int phase, int leg){    // Gets appropriate alfa
  int index = gaitAngleIndices[phase][leg];
  int sign;
  if(gait == STRAIGHT) sign = 1;
  if(gait == TURN) sign = -1;
    switch (index){
      case 0: return -sign*alfa2;
      case 1: return -sign*alfa1;
      case 2: return 0;
      case 3: return sign*alfa1;
      case 4: return sign*alfa2;
      }
  }

// Used in the for(phase...) loop in loop()
boolean testPhase(int phase){  // See if the value of phase is valid
  if (initialPhase == 0){             // Walking forward
    if (phase < nPhases) return true;
    return false;
    }
  else{ if(phase >= 0) return true;}  // Walking backward, need phase >= 0
  return false;
  }

#ifdef usePS2
void getInput(){    // Get and process data from PS2 controller
  input = readPS2(true);    // (In PS2.cpp) Read PS2 controller(true -> debug; print results)
  if(input == PSB_TRIANGLE){  // Reverse direction
    if(initialPhase == 0) initialPhase = nPhases - 1;
    else initialPhase = 0;
    deltaPhase = - deltaPhase;
    }
  if(input == PSB_SQUARE){  // Reduce speed (Increase time)
    if(phaseTime < maxTime) phaseTime = (int)(phaseTime * timeFactor);
    }
  if(input == PSB_CIRCLE){  // Increase speed (Reduce time)
    if(phaseTime > minTime) phaseTime = (int)(phaseTime / timeFactor);
    }
  if(input == PSB_PAD_UP || input == PSB_PAD_DOWN){  // gait = walk STRAIGHT
    gait = STRAIGHT;
    }
  if(input == PSB_PAD_LEFT){  // gait  = TURN
    gait = TURN;
    }
  if(input == PSB_START){  // gait  = TURN
    start = !start;    // Toggle start
    }
  }
#endif

// Computes servo settings to put legs at specified locations
void InverseKinematics(int phase, int leg, float alfa, leg_servo_inputs *legPtr){
// alfa = hipHorizontal rotation (Horizontal motion)
// theta1 = hipVertical rotation (Vertical motion)
// theta2 = knee rotation (Vertical motion)
// They are zero at the calibrated servo positions
   float D, R, Rsq, theta1, theta2, phi, beta, zeta, theta1PlusBeta;
   int alfaSign, otherSign;
   if(gait == STRAIGHT){
     if( leg < 3){ alfaSign = -1; otherSign = -1;}  // Right side of robot
     else{ alfaSign = 1; otherSign = 1;}        // Left side of robot
     }
   if(gait == TURN){
     if( leg < 3){ alfaSign = 1; otherSign = -1;}  // Right side of robot
     else{ alfaSign = 1; otherSign = 1;}        // Left side of robot
     }
   legPtr->hip_H = alfaSign*(int)(alfa*muSecPerRad);
   if(gait_positions[phase][leg] == dn){  //Do IK only for down legs
     float h;
     if (phase != 4){ h = H + deltaH;}  // Adjust for servo sag when on three legs
     else {h = H;}
     D = calculateD(alfa);
     Rsq = H*H + (D - L0)*(D - L0);
     R = sqrt(Rsq);
     zeta = acos(H/R);
     beta = pi/2 - zeta;

     phi = acos( (L1*L1 + L2*L2 - Rsq)/(2*L1*L2) );  // Use law of cosines
     theta1PlusBeta = acos( (L1*L1 + Rsq - L2*L2)/(2*L1*R) );  //theta1 plus beta
     theta1 = theta1PlusBeta - beta;
     theta2 = (pi - phi) - pi/2;  // Want theta2 to be zero when (pi - phi) = reference value(pi/2)
     legPtr->hip_V = otherSign*(int)(theta1*muSecPerRad);
     legPtr->knee = -otherSign*(int)(theta2*muSecPerRad);
     }
   else{  // No IK for legs that are up
     legPtr->hip_V = -otherSign*(int)(liftAngle*radPerDeg*muSecPerRad);
     legPtr->knee = -otherSign*(int)(liftAngle*radPerDeg*muSecPerRad);
     }
   }

void setup(){
  delay(5000);    // 5 sec delay for turning on Serial Monitor
  SSCSerial.begin(38400);  // Enable SSCSerial
  Serial.begin(57600);     // Enable Serial for testing
#ifdef usePS2
  setupPS2();
//  attachInterrupt(interruptOnPin_2, getInput, CHANGE);  // This does not work
//  attachInterrupt(interruptOnPin_3, getInput, CHANGE);  // This does not work
#endif
  alfaSTRAIGHT = gammaSTRAIGHT*radPerDeg;          // Radians
  alfaTURN = atan(R*sin(gammaTURN*radPerDeg)/d);   // Radians
  FindServoInitialSettings();
  startTime = millis();  // Start measuring time for possible need for delay
  }

void loop(){
#ifdef usePS2
  getInput();  // Get commands from PS2 controller once per complete step
#endif
  if(start){
    if(gait == STRAIGHT){    // Note that alfa1 and alfa2 are in radians
      alfa2 = alfaSTRAIGHT;
      alfa1 = (atan(alfaSTRAIGHT)/2);  // For 2 equal-size steps for alfa1 & alfa2
      }
    if(gait == TURN){
      alfa2 = alfaTURN;
      alfa1 = (atan(alfaTURN)/2);  // For 2 equal-size steps for alfa1 & alfa2
      }
    if(initialPhase == 0) deltaPhase = 1; else deltaPhase = -1; // Values for walking forward, else ...
    for(int phase = initialPhase; testPhase(phase); phase = phase + deltaPhase){
      for(int legNo = 0; legNo < nLegs; legNo++){ // Create commands for each leg for current phase
        InverseKinematics(phase, legNo, getAlfa(phase, legNo), &leg[legNo]);
        buildCommand(legNo, 0, leg[legNo].hip_H); // Hip horizontal-motion servo
        buildCommand(legNo, 1, leg[legNo].hip_V); // Hip vertical-motion servo
        buildCommand(legNo, 2, leg[legNo].knee);  // Knee servo
        }  // End of for-each-leg loop   
      previousTime = startTime;
      currentTime = millis();          // Get the current time for the next phase
      computeTime = currentTime - previousTime; // computeTime is time spent doing calculations
      if(phaseTime > computeTime) extraTime = phaseTime - computeTime;
      else extraTime = 0;
      // Execute the command for all the legs for the current phase
      if( extraTime > 0 ) delay(extraTime); // Delay only if there is left-over time
      SSCSerial.print(" T");     // Time for the servo movement
      SSCSerial.println(phaseTime);   // Complete the command, ms
      startTime = millis();          // Get the starting time for the next phase
      }   // End of a phase
    }     // End of if(start) clause
  }     // End of a loop

